/**
 * Copyright (c) 2010 Alois Glomann gnalff@lucubrators.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.lucubrators.honeycomb.defaultimpl.engine;

import java.lang.annotation.Annotation;
import java.util.List;

import net.lucubrators.honeycomb.core.bind.Binder;
import net.lucubrators.honeycomb.core.constraint.Constraint;
import net.lucubrators.honeycomb.core.controller.FormObjectHolder;
import net.lucubrators.honeycomb.core.controller.FormObjectProvider;
import net.lucubrators.honeycomb.core.controller.HandleEvent;
import net.lucubrators.honeycomb.core.controller.Populate;
import net.lucubrators.honeycomb.core.engine.Bindables;
import net.lucubrators.honeycomb.core.reflect.exceptions.CombInvocationException;
import net.lucubrators.honeycomb.defaultimpl.reflect.ArgumentResolver;
import net.lucubrators.honeycomb.defaultimpl.reflect.MethodArgument;
import net.lucubrators.honeycomb.defaultimpl.reflect.MethodHolder;
import net.lucubrators.honeycomb.defaultimpl.reflect.MethodInvoker;
import net.lucubrators.honeycomb.defaultimpl.reflect.MethodResolver;

public class DefaultObjectInvoker implements ObjectInvoker{

	private Object invoke(Class<? extends Annotation> annotation, Object o, Bindables bindables) throws CombInvocationException {
		MethodHolder<? extends Annotation> resolveMethod = methodResolver.resolveMethod(o, annotation);
		List<MethodArgument> resolveArguments = argumentResolver.resolveArguments(resolveMethod);
		Object[] ret = new Object[resolveArguments.size()];
		for (int i = 0; i < resolveArguments.size(); i++) {
			ret[i] = bindingMapper.map(resolveArguments.get(i), bindables);
		}
		return methodInvoker.invokeMethod(resolveMethod, ret);
	}

	@Override
	public Object invokeConstraint(Object constraint, Bindables bindables) throws CombInvocationException {
		return invoke(Constraint.class, constraint, bindables);
	}

	@Override
	public void invokeBinder(Object binder, Bindables bindables) throws CombInvocationException {
		invoke(Binder.class, binder, bindables);
	}

	@Override
	public boolean hasMethodAnnotatedWith(Object controller, Class<? extends Annotation> annotation) {
		MethodHolder<? extends Annotation> resolveMethod = methodResolver.resolveMethod(controller, annotation);
		return resolveMethod != null;
	}

	@Override
	public Object doGet(Object controller, Bindables bindables) throws CombInvocationException {
		return invoke(Populate.class, controller, bindables);
	}

	@Override
	public Object doPost(Object controller, Bindables bindables) throws CombInvocationException {
		return invoke(HandleEvent.class, controller, bindables);
	}

	@Override
	public FormObjectHolder getFormObject(Object controller, Bindables bindables) throws CombInvocationException {
		MethodHolder<FormObjectProvider> resolveMethod = methodResolver.resolveMethod(controller, FormObjectProvider.class);
		Object invoke = invoke(FormObjectProvider.class, controller, bindables);
		FormObjectProvider annotation = resolveMethod.getAnnotation();
		return new FormObjectHolder(invoke, annotation.name(), annotation.binder(), annotation.dontBind());
	}

	private MethodResolver methodResolver;
	private ArgumentResolver argumentResolver;
	private MethodInvoker methodInvoker;
	private BindingMapper bindingMapper;

	public void setMethodInvoker(MethodInvoker methodInvoker) {
		this.methodInvoker = methodInvoker;
	}

	public void setArgumentResolver(ArgumentResolver argumentResolver) {
		this.argumentResolver = argumentResolver;
	}

	public void setMethodResolver(MethodResolver methodResolver) {
		this.methodResolver = methodResolver;
	}

	public void setBindingMapper(BindingMapper bindingMapper) {
		this.bindingMapper = bindingMapper;
	}
}
