/**
 * Copyright (c) 2010 Alois Glomann gnalff@lucubrators.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.lucubrators.honeycomb.defaultimpl.engine;

import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import net.lucubrators.honeycomb.core.Comb;
import net.lucubrators.honeycomb.core.engine.SubmittingCombResolver;

public class DefaultSubmittingCombResolver implements SubmittingCombResolver {

	public static final String DEFAULT_REQUEST_PARAMTER_NAME = "_event";

	private final String eventParamterName;

	public DefaultSubmittingCombResolver() {
		this(DEFAULT_REQUEST_PARAMTER_NAME);
	}

	public DefaultSubmittingCombResolver(String eventParamterName) {
		this.eventParamterName = eventParamterName;
	}

	@Override
	public Comb findSubmittingComb(Comb root, HttpServletRequest request) {
		String parameter = request.getParameter(eventParamterName);
		if (parameter == null || parameter.isEmpty()) {
			return null;
		}
		if (parameter.equals(root.getEventName())) {
			return root;
		}
		Set<Entry<String, List<Comb>>> entrySet = root.getMarks().entrySet();
		for (Entry<String, List<Comb>> entry : entrySet) {
			List<Comb> subCombs = entry.getValue();
			for (Comb subComb : subCombs) {
				Comb findSubmittingComb = findSubmittingComb(subComb, request);
				if (findSubmittingComb != null) {
					return findSubmittingComb;
				}
			}
		}
		return null;
	}

}
