/**
 * Copyright (c) 2010 Alois Glomann gnalff@lucubrators.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.lucubrators.honeycomb.defaultimpl.reflect;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

/**
 * Default implementation of an {@link ArgumentResolver}
 * 
 * @author gnalFF
 * 
 */
public class DefaultArgumentResolver implements ArgumentResolver {

	private ConcurrentHashMap<MethodHolder<? extends Annotation>, List<MethodArgument>> cache;

	public DefaultArgumentResolver() {
		cache = new ConcurrentHashMap<MethodHolder<? extends Annotation>, List<MethodArgument>>();
	}

	@Override
	public List<MethodArgument> resolveArguments(final MethodHolder<? extends Annotation> methodHolder) {
		if (cache.containsKey(methodHolder)) {
			return cache.get(methodHolder);
		}
		Method method = methodHolder.getMethod();

		Type[] genericTypes = method.getGenericParameterTypes();
		Class<?>[] parameterTypes = method.getParameterTypes();
		ArrayList<MethodArgument> ret = new ArrayList<MethodArgument>(parameterTypes.length);

		Annotation[][] parameterAnnotations = method.getParameterAnnotations();
		for (int i = 0; i < parameterTypes.length; i++) {
			ret.add(new MethodArgument(parameterTypes[i], genericTypes[i], Arrays.asList(parameterAnnotations[i])));
		}
		List<MethodArgument> cachedResult = cache.putIfAbsent(methodHolder, ret);
		if (cachedResult == null) {
			return ret;
		}
		return cachedResult;
	}
}
