/**
 * Copyright (c) 2010 Alois Glomann gnalff@lucubrators.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.lucubrators.honeycomb.defaultimpl.util;

import java.lang.reflect.GenericArrayType;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.lang.reflect.TypeVariable;
import java.lang.reflect.WildcardType;

/**
 * Reflects generic java types
 * 
 * @author gnalFF
 * 
 */
public final class GenericType {

	public static GenericType valueOf(final Type type) {
		return new GenericType(type);
	}

	private Type type;

	private GenericType(final Type type) {
		this.type = type;
	}

	public Type getRawType() {
		return type;
	}

	public boolean isClass() {
		return (this.type instanceof Class);
	}

	public boolean isParamterizedType() {
		return (this.type instanceof ParameterizedType);
	}

	public boolean isTypeVariable() {
		return (this.type instanceof TypeVariable);
	}

	public boolean isGenericArrayType() {
		return (this.type instanceof GenericArrayType);
	}

	public boolean isWildcardType() {
		return (this.type instanceof WildcardType);
	}

	public Class<?> typeToClass() {
		return (Class<?>) type;
	}

	public ParameterizedType toParameterizedType() {
		return (ParameterizedType) type;
	}

	public TypeVariable<?> toTypeVariable() {
		return (TypeVariable<?>) type;
	}

	public GenericArrayType toGenericArrayType() {
		return (GenericArrayType) type;
	}

	public WildcardType toWildcardType() {
		return (WildcardType) type;
	}

	@Override
	public String toString() {
		return "ReflectionType [type=" + type + "]";
	}

}
