/**
 * Copyright (c) 2010 Alois Glomann gnalff@lucubrators.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.lucubrators.honeycomb.defaultimpl.web.tag;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import net.lucubrators.honeycomb.core.controller.ControllerResult;
import net.lucubrators.honeycomb.core.util.ControllerResultUtil;

/**
 * Base Tag providing some convinience methods
 * 
 * @author gnalFF
 * 
 */
public abstract class BaseCombTag extends SimpleTagSupport {

	@Override
	public void doTag() throws JspException, IOException {
		ControllerResult currentResult = getCurrentControllerResult();
		if (currentResult == null) {
			throw new IllegalStateException("CurrentResult is empty. Did you forget to initialize?");
		}
		internalDoTag();
	}

	/**
	 * 
	 * @return the current controller result to be handled. never null
	 */
	public ControllerResult getCurrentControllerResult() {
		ControllerResult currentResult = ControllerResultUtil.getCurrentResult(getRequest());
		if (currentResult == null) {
			throw new IllegalStateException("No current result. Did you forget to set the current result");
		}
		return currentResult;
	}

	/**
	 * @return {@link JspWriter}
	 */
	public JspWriter getWriter() {
		return getJspContext().getOut();
	}
	
	/**
	 * 
	 * @return the current HttpServletRequest
	 */
	public HttpServletRequest getRequest() {
		return (HttpServletRequest) ((PageContext) getJspContext()).getRequest();
	}

	/**
	 * Override this method to handle your tag
	 * 
	 * @throws JspException
	 * @throws IOException
	 */
	public abstract void internalDoTag() throws JspException, IOException;

}
