/**
 * Copyright (c) 2010 Alois Glomann gnalff@lucubrators.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.lucubrators.honeycomb.defaultimpl.web.tag;

import java.io.IOException;
import java.util.List;

import javax.servlet.jsp.JspException;

import net.lucubrators.honeycomb.core.controller.ControllerResult;
import net.lucubrators.honeycomb.core.messages.Message;
import net.lucubrators.honeycomb.core.messages.MessageLevel;

/**
 * Print errors which happened while binding
 * 
 * @author gnalFF
 * 
 */
public class DisplayMessagesTag extends BaseCombTag {

	private String	level;
	private String	containerClass;
	private String	messageClass;

	@Override
	public void internalDoTag() throws JspException, IOException {
		ControllerResult currentResult = getCurrentControllerResult();
		MessageLevel messageLevel = MessageLevel.valueOf(level!=null?level.toUpperCase():level);

		List<Message> messagesByLevel = currentResult.getMessages().getMessagesByLevel(messageLevel);
		if (messagesByLevel.isEmpty()) {
			return;
		}
		if (null == containerClass || containerClass.isEmpty()) {
			setContainerClass(messageLevel.name().toLowerCase() + "s");
		}
		if (null == messageClass || messageClass.isEmpty()) {
			setMessageClass(messageLevel.name().toLowerCase());
		}

		getWriter().write("<div class=\"" + getContainerClass() + "\">");
		for (Message message : messagesByLevel) {
			getWriter().write("<p class=\"" + getMessageClass() + "\">");
			getWriter().write(message.getMessage());
			getWriter().write("</p>");
		}
		getWriter().write("</div>");
	}

	public String getLevel() {
		return level;
	}

	public void setLevel(final String level) {
		this.level = level;
	}

	public String getContainerClass() {
		return containerClass;
	}

	public void setContainerClass(final String containerClass) {
		this.containerClass = containerClass;
	}

	public String getMessageClass() {
		return messageClass;
	}

	public void setMessageClass(final String messageClass) {
		this.messageClass = messageClass;
	}
}
