/**
 * Copyright (c) 2010 Alois Glomann gnalff@lucubrators.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.lucubrators.honeycomb.defaultimpl.web.tag;

import java.io.IOException;
import java.util.List;

import javax.servlet.jsp.JspException;

import net.lucubrators.honeycomb.core.controller.ControllerResult;
import net.lucubrators.honeycomb.core.messages.ErrorMessage;
import net.lucubrators.honeycomb.core.messages.Message;
import net.lucubrators.honeycomb.core.messages.MessageLevel;

/**
 * Tag which returns the value or the error causing value
 * 
 * @author gnalFF
 * 
 */
public class ErrorOrValueTag extends BaseCombTag {

	private String	propertyName;
	private String	currentValue;

	@Override
	public void internalDoTag() throws JspException, IOException {
		ControllerResult currentResult = getCurrentControllerResult();
		List<Message> messages = currentResult.getMessages().getMessagesByLevel(MessageLevel.ERROR);
		for (Message message : messages) {
			ErrorMessage err = (ErrorMessage) message;
			if (err.getProperty().equals(propertyName) && err.hasErrorValue()) {
				getWriter().write(String.valueOf(err.getErrorValue()));
				return;
			}
		}
		getWriter().write(getCurrentValue());
	}

	public void setPropertyName(final String propertyName) {
		this.propertyName = propertyName;
	}

	public String getPropertyName() {
		return propertyName;
	}

	public void setCurrentValue(final String currentValue) {
		this.currentValue = currentValue;
	}

	public String getCurrentValue() {
		return currentValue;
	}
}
